// -*- mode: C++ -*-

// AUTOGENERATED BY glean_parser.  DO NOT EDIT.

/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#ifndef mozilla_GleanJSMetricsLookup_h
#define mozilla_GleanJSMetricsLookup_h

#include "mozilla/PerfectHash.h"
#include "mozilla/Maybe.h"
#include "mozilla/glean/MetricTypes.h"

#define GLEAN_INDEX_BITS (32)
#define GLEAN_ID_BITS (27)
#define GLEAN_TYPE_ID(id) ((id) >> GLEAN_ID_BITS)
#define GLEAN_METRIC_ID(id) ((id) & ((1ULL << GLEAN_ID_BITS) - 1))
#define GLEAN_OFFSET(entry) (entry & ((1ULL << GLEAN_INDEX_BITS) - 1))

namespace mozilla::glean {

// The category lookup table's entry type
using category_entry_t = uint32_t;
// The metric lookup table's entry type
using metric_entry_t = uint64_t;

static_assert(GLEAN_INDEX_BITS + GLEAN_ID_BITS < sizeof(metric_entry_t) * 8, "Index and ID bits need to fit into an category_entry_t");
static_assert(GLEAN_ID_BITS < sizeof(uint32_t) * 8, "Metric IDs need to fit into less than 32 bit");
static_assert(5 < UINT32_MAX, "Too many metric categories generated.");
static_assert(16 < 134217728, "Too many metrics generated.");
static_assert(10 < 32, "Too many different metric types.");

static already_AddRefed<nsISupports> NewMetricFromId(uint32_t id) {
  uint32_t typeId = GLEAN_TYPE_ID(id);
  uint32_t metricId = GLEAN_METRIC_ID(id);

  switch (typeId) {
    case 1: /* Timespan */
    {
      RefPtr<nsISupports> metric = new GleanTimespan(metricId);
      return metric.forget();
      break;
    }
    case 2: /* Counter */
    {
      RefPtr<nsISupports> metric = new GleanCounter(metricId);
      return metric.forget();
      break;
    }
    case 3: /* Uuid */
    {
      RefPtr<nsISupports> metric = new GleanUuid(metricId);
      return metric.forget();
      break;
    }
    case 4: /* String */
    {
      RefPtr<nsISupports> metric = new GleanString(metricId);
      return metric.forget();
      break;
    }
    default:
    MOZ_ASSERT_UNREACHABLE("Invalid type ID reached when trying to instantiate a new metric");
    return nullptr;
  }
}

static Maybe<uint32_t> category_result_check(const nsACString& aKey, category_entry_t entry);
static Maybe<uint32_t> metric_result_check(const nsACString& aKey, metric_entry_t entry);

#if defined(_MSC_VER) && !defined(__clang__)
const char gCategoryStringTable[] = {
#else
constexpr char gCategoryStringTable[] = {
#endif
  /*     0 - "fog" */ 'f', 'o', 'g', '\0',
  /*     4 - "fog_ipc" */ 'f', 'o', 'g', '_', 'i', 'p', 'c', '\0',
  /*    12 - "fog_validation" */ 'f', 'o', 'g', '_', 'v', 'a', 'l', 'i', 'd', 'a', 't', 'i', 'o', 'n', '\0',
  /*    27 - "test_only" */ 't', 'e', 's', 't', '_', 'o', 'n', 'l', 'y', '\0',
  /*    37 - "test_only_ipc" */ 't', 'e', 's', 't', '_', 'o', 'n', 'l', 'y', '_', 'i', 'p', 'c', '\0',
};


static_assert(sizeof(gCategoryStringTable) < UINT32_MAX, "Category string table is too large.");

const category_entry_t sCategoryByNameLookupEntries[] = {
  27,
  12,
  0,
  37,
  4
};



static Maybe<uint32_t>
CategoryByNameLookup(const nsACString& aKey)
{
  static const uint8_t BASES[] = {
       0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   1,   0,   0,
       0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
       0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
       0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   3,
  };
  

  const char* bytes = aKey.BeginReading();
  size_t length = aKey.Length();
  auto& entry = mozilla::perfecthash::Lookup(bytes, length, BASES,
                                             sCategoryByNameLookupEntries);
  return category_result_check(aKey, entry);
}


#if defined(_MSC_VER) && !defined(__clang__)
const char gMetricStringTable[] = {
#else
constexpr char gMetricStringTable[] = {
#endif
  /*     0 - "fog.initialization" */ 'f', 'o', 'g', '.', 'i', 'n', 'i', 't', 'i', 'a', 'l', 'i', 'z', 'a', 't', 'i', 'o', 'n', '\0',
  /*    19 - "fog_ipc.replay_failures" */ 'f', 'o', 'g', '_', 'i', 'p', 'c', '.', 'r', 'e', 'p', 'l', 'a', 'y', '_', 'f', 'a', 'i', 'l', 'u', 'r', 'e', 's', '\0',
  /*    43 - "fog_validation.legacy_telemetry_client_id" */ 'f', 'o', 'g', '_', 'v', 'a', 'l', 'i', 'd', 'a', 't', 'i', 'o', 'n', '.', 'l', 'e', 'g', 'a', 'c', 'y', '_', 't', 'e', 'l', 'e', 'm', 'e', 't', 'r', 'y', '_', 'c', 'l', 'i', 'e', 'n', 't', '_', 'i', 'd', '\0',
  /*    85 - "test_only.bad_code" */ 't', 'e', 's', 't', '_', 'o', 'n', 'l', 'y', '.', 'b', 'a', 'd', '_', 'c', 'o', 'd', 'e', '\0',
  /*   104 - "test_only.can_we_time_it" */ 't', 'e', 's', 't', '_', 'o', 'n', 'l', 'y', '.', 'c', 'a', 'n', '_', 'w', 'e', '_', 't', 'i', 'm', 'e', '_', 'i', 't', '\0',
  /*   129 - "test_only.cheesy_string" */ 't', 'e', 's', 't', '_', 'o', 'n', 'l', 'y', '.', 'c', 'h', 'e', 'e', 's', 'y', '_', 's', 't', 'r', 'i', 'n', 'g', '\0',
  /*   153 - "test_only.what_id_it" */ 't', 'e', 's', 't', '_', 'o', 'n', 'l', 'y', '.', 'w', 'h', 'a', 't', '_', 'i', 'd', '_', 'i', 't', '\0',
  /*   174 - "test_only_ipc.a_counter" */ 't', 'e', 's', 't', '_', 'o', 'n', 'l', 'y', '_', 'i', 'p', 'c', '.', 'a', '_', 'c', 'o', 'u', 'n', 't', 'e', 'r', '\0',
  /*   198 - "test_only_ipc.a_bool" */ 't', 'e', 's', 't', '_', 'o', 'n', 'l', 'y', '_', 'i', 'p', 'c', '.', 'a', '_', 'b', 'o', 'o', 'l', '\0',
  /*   219 - "test_only_ipc.a_date" */ 't', 'e', 's', 't', '_', 'o', 'n', 'l', 'y', '_', 'i', 'p', 'c', '.', 'a', '_', 'd', 'a', 't', 'e', '\0',
  /*   240 - "test_only_ipc.a_string" */ 't', 'e', 's', 't', '_', 'o', 'n', 'l', 'y', '_', 'i', 'p', 'c', '.', 'a', '_', 's', 't', 'r', 'i', 'n', 'g', '\0',
  /*   263 - "test_only_ipc.a_memory_dist" */ 't', 'e', 's', 't', '_', 'o', 'n', 'l', 'y', '_', 'i', 'p', 'c', '.', 'a', '_', 'm', 'e', 'm', 'o', 'r', 'y', '_', 'd', 'i', 's', 't', '\0',
  /*   291 - "test_only_ipc.a_timing_dist" */ 't', 'e', 's', 't', '_', 'o', 'n', 'l', 'y', '_', 'i', 'p', 'c', '.', 'a', '_', 't', 'i', 'm', 'i', 'n', 'g', '_', 'd', 'i', 's', 't', '\0',
  /*   319 - "test_only_ipc.a_string_list" */ 't', 'e', 's', 't', '_', 'o', 'n', 'l', 'y', '_', 'i', 'p', 'c', '.', 'a', '_', 's', 't', 'r', 'i', 'n', 'g', '_', 'l', 'i', 's', 't', '\0',
  /*   347 - "test_only_ipc.an_event" */ 't', 'e', 's', 't', '_', 'o', 'n', 'l', 'y', '_', 'i', 'p', 'c', '.', 'a', 'n', '_', 'e', 'v', 'e', 'n', 't', '\0',
  /*   370 - "test_only_ipc.a_uuid" */ 't', 'e', 's', 't', '_', 'o', 'n', 'l', 'y', '_', 'i', 'p', 'c', '.', 'a', '_', 'u', 'u', 'i', 'd', '\0',
};


static_assert(sizeof(gMetricStringTable) < 4294967296, "Metric string table is too large.");

const metric_entry_t sMetricByNameLookupEntries[] = {
  1729382269795172395,
  2882303800171823302,
  5764607587458744667,
  576460756598390784,
  4611686074261963043,
  5188146830860353855,
  1729382325629747570,
  1152921521786716245,
  576460773778260072,
  2305843034983497857,
  1152921513196781587,
  1729382286975041689,
  2305843056458334448,
  3458764556770214107,
  1152921538966585518,
  4035225317663572231
};



static Maybe<uint32_t>
MetricByNameLookup(const nsACString& aKey)
{
  static const uint8_t BASES[] = {
       0,   0,   1,   1,   0,   0,   0,   0,   0,   1,   0,   2,   0,   1,   3,   0,
       0,   1,   0,   0,   0,   2,   0,   0,   2,   0,   0,   0,   0,   0,   0,   0,
       0,   0,   0,   1,   0,   0,   0,   1,   1,   0,   0,   0,   0,   0,   0,   0,
       0,   0,   0,   6,   0,   1,   0,   0,   0,   8,   0,   0,   0,   0,   0,   0,
  };
  

  const char* bytes = aKey.BeginReading();
  size_t length = aKey.Length();
  auto& entry = mozilla::perfecthash::Lookup(bytes, length, BASES,
                                             sMetricByNameLookupEntries);
  return metric_result_check(aKey, entry);
}


/**
 * Get a category's name from the string table.
 */
static const char* GetCategoryName(category_entry_t entry) {
  MOZ_ASSERT(entry < sizeof(gCategoryStringTable), "Entry identifier offset larger than string table");
  return &gCategoryStringTable[entry];
}

/**
 * Get a metric's identifier from the string table.
 */
static const char* GetMetricIdentifier(metric_entry_t entry) {
  uint32_t offset = GLEAN_OFFSET(entry);
  MOZ_ASSERT(offset < sizeof(gMetricStringTable), "Entry identifier offset larger than string table");
  return &gMetricStringTable[offset];
}

/**
 * Check that the found entry is pointing to the right key
 * and return it.
 * Or return `Nothing()` if the entry was not found.
 */
static Maybe<uint32_t> category_result_check(const nsACString& aKey, category_entry_t entry) {
  if (MOZ_UNLIKELY(entry > sizeof(gCategoryStringTable))) {
    return Nothing();
  }
  if (aKey.EqualsASCII(gCategoryStringTable + entry)) {
    return Some(entry);
  }
  return Nothing();
}

/**
 * Check if the found entry index is pointing to the right key
 * and return the corresponding metric ID.
 * Or return `Nothing()` if the entry was not found.
 */
static Maybe<uint32_t> metric_result_check(const nsACString& aKey, uint64_t entry) {
  uint32_t metricId = entry >> GLEAN_INDEX_BITS;
  uint32_t offset = GLEAN_OFFSET(entry);

  if (offset > sizeof(gMetricStringTable)) {
    return Nothing();
  }

  if (aKey.EqualsASCII(gMetricStringTable + offset)) {
    return Some(metricId);
  }

  return Nothing();
}


#undef GLEAN_INDEX_BITS
#undef GLEAN_ID_BITS
#undef GLEAN_TYPE_ID
#undef GLEAN_METRIC_ID
#undef GLEAN_OFFSET

} // namespace mozilla::glean
#endif // mozilla_GleanJSMetricsLookup_h
